"""
Python module generated from Java source file com.google.common.collect.Lists

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.annotations import VisibleForTesting
from com.google.common.base import Function
from com.google.common.base import Objects
from com.google.common.collect import *
from com.google.common.math import IntMath
from com.google.common.primitives import Ints
from java.io import Serializable
from java.math import RoundingMode
from java.util import AbstractList
from java.util import AbstractSequentialList
from java.util import Arrays
from java.util import Collections
from java.util import Iterator
from java.util import ListIterator
from java.util import NoSuchElementException
from java.util import RandomAccess
from java.util.concurrent import CopyOnWriteArrayList
from java.util.function import Predicate
from javax.annotation import CheckForNull
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Lists:
    """
    Static utility methods pertaining to List instances. Also see this class's counterparts
    Sets, Maps and Queues.
    
    See the Guava User Guide article on <a href=
    "https://github.com/google/guava/wiki/CollectionUtilitiesExplained#lists">`Lists`</a>.

    Author(s)
    - Louis Wasserman

    Since
    - 2.0
    """

    @staticmethod
    def newArrayList() -> list["E"]:
        """
        Creates a *mutable*, empty `ArrayList` instance (for Java 6 and earlier).
        
        **Note:** if mutability is not required, use ImmutableList.of() instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `ArrayList` ArrayList.ArrayList() constructor directly, taking
        advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        """
        ...


    @staticmethod
    def newArrayList(*elements: Tuple["E", ...]) -> list["E"]:
        """
        Creates a *mutable* `ArrayList` instance containing the given elements.
        
        **Note:** essentially the only reason to use this method is when you will need to add or
        remove elements later. Otherwise, for non-null elements use ImmutableList.of() (for
        varargs) or ImmutableList.copyOf(Object[]) (for an array) instead. If any elements
        might be null, or you need support for List.set(int, Object), use Arrays.asList.
        
        Note that even when you do need the ability to add or remove, this method provides only a
        tiny bit of syntactic sugar for `newArrayList(`Arrays.asList asList`(...))`, or for creating an empty list then calling Collections.addAll. This method is
        not actually very useful and will likely be deprecated in the future.
        """
        ...


    @staticmethod
    def newArrayList(elements: Iterable["E"]) -> list["E"]:
        """
        Creates a *mutable* `ArrayList` instance containing the given elements; a very thin
        shortcut for creating an empty list then calling Iterables.addAll.
        
        **Note:** if mutability is not required and the elements are non-null, use ImmutableList.copyOf(Iterable) instead. (Or, change `elements` to be a FluentIterable and call `elements.toList()`.)
        
        **Note:** if `elements` is a Collection, you don't need this method. Use
        the `ArrayList` ArrayList.ArrayList(Collection) constructor directly, taking
        advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        """
        ...


    @staticmethod
    def newArrayList(elements: Iterator["E"]) -> list["E"]:
        """
        Creates a *mutable* `ArrayList` instance containing the given elements; a very thin
        shortcut for creating an empty list and then calling Iterators.addAll.
        
        **Note:** if mutability is not required and the elements are non-null, use ImmutableList.copyOf(Iterator) instead.
        """
        ...


    @staticmethod
    def newArrayListWithCapacity(initialArraySize: int) -> list["E"]:
        """
        Creates an `ArrayList` instance backed by an array with the specified initial size;
        simply delegates to ArrayList.ArrayList(int).
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use `new`ArrayList.ArrayList(int) ArrayList`<>(int)` directly, taking
        advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>. (Unlike here, there is no risk of overload ambiguity, since the `ArrayList`
        constructors very wisely did not accept varargs.)

        Arguments
        - initialArraySize: the exact size of the initial backing array for the returned array list
            (`ArrayList` documentation calls this value the "capacity")

        Returns
        - a new, empty `ArrayList` which is guaranteed not to resize itself unless its size
            reaches `initialArraySize + 1`

        Raises
        - IllegalArgumentException: if `initialArraySize` is negative
        """
        ...


    @staticmethod
    def newArrayListWithExpectedSize(estimatedSize: int) -> list["E"]:
        """
        Creates an `ArrayList` instance to hold `estimatedSize` elements, *plus* an
        unspecified amount of padding; you almost certainly mean to call .newArrayListWithCapacity (see that method for further advice on usage).
        
        **Note:** This method will soon be deprecated. Even in the rare case that you do want
        some amount of padding, it's best if you choose your desired amount explicitly.

        Arguments
        - estimatedSize: an estimate of the eventual List.size() of the new list

        Returns
        - a new, empty `ArrayList`, sized appropriately to hold the estimated number of
            elements

        Raises
        - IllegalArgumentException: if `estimatedSize` is negative
        """
        ...


    @staticmethod
    def newLinkedList() -> list["E"]:
        """
        Creates a *mutable*, empty `LinkedList` instance (for Java 6 and earlier).
        
        **Note:** if you won't be adding any elements to the list, use ImmutableList.of()
        instead.
        
        **Performance note:** ArrayList and java.util.ArrayDeque consistently
        outperform `LinkedList` except in certain rare and specific situations. Unless you have
        spent a lot of time benchmarking your specific needs, use one of those instead.
        
        **Note:** this method is now unnecessary and should be treated as deprecated. Instead,
        use the `LinkedList` LinkedList.LinkedList() constructor directly, taking
        advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        """
        ...


    @staticmethod
    def newLinkedList(elements: Iterable["E"]) -> list["E"]:
        """
        Creates a *mutable* `LinkedList` instance containing the given elements; a very thin
        shortcut for creating an empty list then calling Iterables.addAll.
        
        **Note:** if mutability is not required and the elements are non-null, use ImmutableList.copyOf(Iterable) instead. (Or, change `elements` to be a FluentIterable and call `elements.toList()`.)
        
        **Performance note:** ArrayList and java.util.ArrayDeque consistently
        outperform `LinkedList` except in certain rare and specific situations. Unless you have
        spent a lot of time benchmarking your specific needs, use one of those instead.
        
        **Note:** if `elements` is a Collection, you don't need this method. Use
        the `LinkedList` LinkedList.LinkedList(Collection) constructor directly,
        taking advantage of <a
        href="https://docs.oracle.com/javase/tutorial/java/generics/genTypeInference.html#type-inference-instantiation">"diamond"
        syntax</a>.
        """
        ...


    @staticmethod
    def newCopyOnWriteArrayList() -> "CopyOnWriteArrayList"["E"]:
        """
        Creates an empty `CopyOnWriteArrayList` instance.
        
        **Note:** if you need an immutable empty List, use Collections.emptyList
        instead.

        Returns
        - a new, empty `CopyOnWriteArrayList`

        Since
        - 12.0
        """
        ...


    @staticmethod
    def newCopyOnWriteArrayList(elements: Iterable["E"]) -> "CopyOnWriteArrayList"["E"]:
        """
        Creates a `CopyOnWriteArrayList` instance containing the given elements.

        Arguments
        - elements: the elements that the list should contain, in order

        Returns
        - a new `CopyOnWriteArrayList` containing those elements

        Since
        - 12.0
        """
        ...


    @staticmethod
    def asList(first: "E", rest: list["E"]) -> list["E"]:
        """
        Returns an unmodifiable list containing the specified first element and backed by the specified
        array of additional elements. Changes to the `rest` array will be reflected in the
        returned list. Unlike Arrays.asList, the returned list is unmodifiable.
        
        This is useful when a varargs method needs to use a signature such as `(Foo firstFoo,
        Foo... moreFoos)`, in order to avoid overload ambiguity or to enforce a minimum argument count.
        
        The returned list is serializable and implements RandomAccess.

        Arguments
        - first: the first element
        - rest: an array of additional elements, possibly empty

        Returns
        - an unmodifiable list containing the specified elements
        """
        ...


    @staticmethod
    def asList(first: "E", second: "E", rest: list["E"]) -> list["E"]:
        """
        Returns an unmodifiable list containing the specified first and second element, and backed by
        the specified array of additional elements. Changes to the `rest` array will be reflected
        in the returned list. Unlike Arrays.asList, the returned list is unmodifiable.
        
        This is useful when a varargs method needs to use a signature such as `(Foo firstFoo,
        Foo secondFoo, Foo... moreFoos)`, in order to avoid overload ambiguity or to enforce a minimum
        argument count.
        
        The returned list is serializable and implements RandomAccess.

        Arguments
        - first: the first element
        - second: the second element
        - rest: an array of additional elements, possibly empty

        Returns
        - an unmodifiable list containing the specified elements
        """
        ...


    @staticmethod
    def cartesianProduct(lists: list[list["B"]]) -> list[list["B"]]:
        """
        Returns every possible list that can be formed by choosing one element from each of the given
        lists in order; the "n-ary <a href="http://en.wikipedia.org/wiki/Cartesian_product">Cartesian
        product</a>" of the lists. For example:
        
        ````Lists.cartesianProduct(ImmutableList.of(
            ImmutableList.of(1, 2),
            ImmutableList.of("A", "B", "C")))````
        
        returns a list containing six lists in the following order:
        
        
          - `ImmutableList.of(1, "A")`
          - `ImmutableList.of(1, "B")`
          - `ImmutableList.of(1, "C")`
          - `ImmutableList.of(2, "A")`
          - `ImmutableList.of(2, "B")`
          - `ImmutableList.of(2, "C")`
        
        
        The result is guaranteed to be in the "traditional", lexicographical order for Cartesian
        products that you would get from nesting for loops:
        
        ````for (B b0 : lists.get(0)) {
          for (B b1 : lists.get(1)) {
            ...
            ImmutableList<B> tuple = ImmutableList.of(b0, b1, ...);
            // operate on tuple`
        }
        }```
        
        Note that if any input list is empty, the Cartesian product will also be empty. If no lists
        at all are provided (an empty list), the resulting Cartesian product has one element, an empty
        list (counter-intuitive, but mathematically consistent).
        
        *Performance notes:* while the cartesian product of lists of size `m, n, p` is a
        list of size `m x n x p`, its actual memory consumption is much smaller. When the
        cartesian product is constructed, the input lists are merely copied. Only as the resulting list
        is iterated are the individual lists created, and these are not retained after iteration.
        
        Type `<B>`: any common base class shared by all axes (often just Object)

        Arguments
        - lists: the lists to choose elements from, in the order that the elements chosen from
            those lists should appear in the resulting lists

        Returns
        - the Cartesian product, as an immutable list containing immutable lists

        Raises
        - IllegalArgumentException: if the size of the cartesian product would be greater than
            Integer.MAX_VALUE
        - NullPointerException: if `lists`, any one of the `lists`, or any element of
            a provided list is null

        Since
        - 19.0
        """
        ...


    @staticmethod
    def cartesianProduct(*lists: Tuple[list["B"], ...]) -> list[list["B"]]:
        """
        Returns every possible list that can be formed by choosing one element from each of the given
        lists in order; the "n-ary <a href="http://en.wikipedia.org/wiki/Cartesian_product">Cartesian
        product</a>" of the lists. For example:
        
        ````Lists.cartesianProduct(ImmutableList.of(
            ImmutableList.of(1, 2),
            ImmutableList.of("A", "B", "C")))````
        
        returns a list containing six lists in the following order:
        
        
          - `ImmutableList.of(1, "A")`
          - `ImmutableList.of(1, "B")`
          - `ImmutableList.of(1, "C")`
          - `ImmutableList.of(2, "A")`
          - `ImmutableList.of(2, "B")`
          - `ImmutableList.of(2, "C")`
        
        
        The result is guaranteed to be in the "traditional", lexicographical order for Cartesian
        products that you would get from nesting for loops:
        
        ````for (B b0 : lists.get(0)) {
          for (B b1 : lists.get(1)) {
            ...
            ImmutableList<B> tuple = ImmutableList.of(b0, b1, ...);
            // operate on tuple`
        }
        }```
        
        Note that if any input list is empty, the Cartesian product will also be empty. If no lists
        at all are provided (an empty list), the resulting Cartesian product has one element, an empty
        list (counter-intuitive, but mathematically consistent).
        
        *Performance notes:* while the cartesian product of lists of size `m, n, p` is a
        list of size `m x n x p`, its actual memory consumption is much smaller. When the
        cartesian product is constructed, the input lists are merely copied. Only as the resulting list
        is iterated are the individual lists created, and these are not retained after iteration.
        
        Type `<B>`: any common base class shared by all axes (often just Object)

        Arguments
        - lists: the lists to choose elements from, in the order that the elements chosen from
            those lists should appear in the resulting lists

        Returns
        - the Cartesian product, as an immutable list containing immutable lists

        Raises
        - IllegalArgumentException: if the size of the cartesian product would be greater than
            Integer.MAX_VALUE
        - NullPointerException: if `lists`, any one of the `lists`, or any element of
            a provided list is null

        Since
        - 19.0
        """
        ...


    @staticmethod
    def transform(fromList: list["F"], function: "Function"["F", "T"]) -> list["T"]:
        """
        Returns a list that applies `function` to each element of `fromList`. The returned
        list is a transformed view of `fromList`; changes to `fromList` will be reflected
        in the returned list and vice versa.
        
        Since functions are not reversible, the transform is one-way and new items cannot be stored
        in the returned list. The `add`, `addAll` and `set` methods are unsupported
        in the returned list.
        
        The function is applied lazily, invoked when needed. This is necessary for the returned list
        to be a view, but it means that the function will be applied many times for bulk operations
        like List.contains and List.hashCode. For this to perform well, `function` should be fast. To avoid lazy evaluation when the returned list doesn't need to be a
        view, copy the returned list into a new list of your choosing.
        
        If `fromList` implements RandomAccess, so will the returned list. The returned
        list is threadsafe if the supplied list and function are.
        
        If only a `Collection` or `Iterable` input is available, use Collections2.transform or Iterables.transform.
        
        **Note:** serializing the returned list is implemented by serializing `fromList`,
        its contents, and `function` -- *not* by serializing the transformed values. This
        can lead to surprising behavior, so serializing the returned list is **not recommended**.
        Instead, copy the list using ImmutableList.copyOf(Collection) (for example), then
        serialize the copy. Other methods similar to this do not implement serialization at all for
        this reason.
        
        **Java 8+ users:** many use cases for this method are better addressed by java.util.stream.Stream.map. This method is not being deprecated, but we gently encourage you
        to migrate to streams.
        """
        ...


    @staticmethod
    def partition(list: list["T"], size: int) -> list[list["T"]]:
        """
        Returns consecutive List.subList(int, int) sublists of a list, each of the same
        size (the final list may be smaller). For example, partitioning a list containing `[a, b,
        c, d, e]` with a partition size of 3 yields `[[a, b, c], [d, e]]` -- an outer list
        containing two inner lists of three and two elements, all in the original order.
        
        The outer list is unmodifiable, but reflects the latest state of the source list. The inner
        lists are sublist views of the original list, produced on demand using List.subList(int,
        int), and are subject to all the usual caveats about modification as explained in that API.

        Arguments
        - list: the list to return consecutive sublists of
        - size: the desired size of each sublist (the last may be smaller)

        Returns
        - a list of consecutive sublists

        Raises
        - IllegalArgumentException: if `partitionSize` is nonpositive
        """
        ...


    @staticmethod
    def charactersOf(string: str) -> "ImmutableList"["Character"]:
        """
        Returns a view of the specified string as an immutable list of `Character` values.

        Since
        - 7.0
        """
        ...


    @staticmethod
    def charactersOf(sequence: "CharSequence") -> list["Character"]:
        """
        Returns a view of the specified `CharSequence` as a `List<Character>`, viewing
        `sequence` as a sequence of Unicode code units. The view does not support any
        modification operations, but reflects any changes to the underlying character sequence.

        Arguments
        - sequence: the character sequence to view as a `List` of characters

        Returns
        - an `List<Character>` view of the character sequence

        Since
        - 7.0
        """
        ...


    @staticmethod
    def reverse(list: list["T"]) -> list["T"]:
        """
        Returns a reversed view of the specified list. For example, `Lists.reverse(Arrays.asList(1, 2, 3))` returns a list containing `3, 2, 1`. The returned
        list is backed by this list, so changes in the returned list are reflected in this list, and
        vice-versa. The returned list supports all of the optional list operations supported by this
        list.
        
        The returned list is random-access if the specified list is random access.

        Since
        - 7.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.help.HelpTopicComparator

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import Comparator
from org.bukkit.help import *
from typing import Any, Callable, Iterable, Tuple


class HelpTopicComparator(Comparator):
    """
    Used to impose a custom total ordering on help topics.
    
    All topics are listed in alphabetic order, but topics that start with a
    slash come after topics that don't.
    """

    @staticmethod
    def topicNameComparatorInstance() -> "TopicNameComparator":
        ...


    @staticmethod
    def helpTopicComparatorInstance() -> "HelpTopicComparator":
        ...


    def compare(self, lhs: "HelpTopic", rhs: "HelpTopic") -> int:
        ...


    class TopicNameComparator(Comparator):

        def compare(self, lhs: str, rhs: str) -> int:
            ...



"""
Python module generated from Java source file com.google.gson.internal.NonNullElementWrapperList

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.internal import *
from java.util import AbstractList
from java.util import Objects
from java.util import RandomAccess
from typing import Any, Callable, Iterable, Tuple


class NonNullElementWrapperList(AbstractList, RandomAccess):
    """
    List which wraps another `List` but prevents insertion of `null` elements.
    Methods which only perform checks with the element argument (e.g. .contains(Object)) do
    not throw exceptions for `null` arguments.
    """

    def __init__(self, delegate: list["E"]):
        ...


    def get(self, index: int) -> "E":
        ...


    def size(self) -> int:
        ...


    def set(self, index: int, element: "E") -> "E":
        ...


    def add(self, index: int, element: "E") -> None:
        ...


    def remove(self, index: int) -> "E":
        ...


    def clear(self) -> None:
        ...


    def remove(self, o: "Object") -> bool:
        ...


    def removeAll(self, c: Iterable[Any]) -> bool:
        ...


    def retainAll(self, c: Iterable[Any]) -> bool:
        ...


    def contains(self, o: "Object") -> bool:
        ...


    def indexOf(self, o: "Object") -> int:
        ...


    def lastIndexOf(self, o: "Object") -> int:
        ...


    def toArray(self) -> list["Object"]:
        ...


    def toArray(self, a: list["T"]) -> list["T"]:
        ...


    def equals(self, o: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerSpawnChangeEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from enum import Enum
from org.bukkit import Location
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.player import *
from typing import Any, Callable, Iterable, Tuple


class PlayerSpawnChangeEvent(PlayerEvent, Cancellable):
    """
    This event is fired when the spawn point of the player is changed.
    """

    def __init__(self, player: "Player", newSpawn: "Location", forced: bool, cause: "Cause"):
        ...


    def getCause(self) -> "Cause":
        """
        Gets the cause of spawn change.

        Returns
        - change cause
        """
        ...


    def isForced(self) -> bool:
        """
        Gets if the spawn position will be used regardless of bed obstruction
        rules.

        Returns
        - True if is forced
        """
        ...


    def setForced(self, forced: bool) -> None:
        """
        Sets if the spawn position will be used regardless of bed obstruction
        rules.

        Arguments
        - forced: True if forced
        """
        ...


    def getNewSpawn(self) -> "Location":
        """
        Gets the new spawn to be set.

        Returns
        - new spawn location
        """
        ...


    def setNewSpawn(self, newSpawn: "Location") -> None:
        """
        Sets the new spawn location.

        Arguments
        - newSpawn: new spawn location, with non-null world
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...


    class Cause(Enum):

        COMMAND = 0
        """
        Indicate the spawn was set by a command.
        """
        BED = 1
        """
        Indicate the spawn was set by the player interacting with a bed.
        """
        RESPAWN_ANCHOR = 2
        """
        Indicate the spawn was set by the player interacting with a respawn
        anchor.
        """
        PLUGIN = 3
        """
        Indicate the spawn was set by the use of plugins.
        """
        RESET = 4
        """
        Indicate the spawn was reset by an invalid bed position or empty
        respawn anchor.
        """
        UNKNOWN = 5
        """
        Indicate the spawn was caused by an unknown reason.
        """



"""
Python module generated from Java source file com.google.common.collect.TreeTraverser

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import Beta
from com.google.common.annotations import GwtCompatible
from com.google.common.base import Function
from com.google.common.collect import *
from java.util import ArrayDeque
from java.util import Deque
from java.util import Iterator
from java.util import Queue
from java.util.function import Consumer
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class TreeTraverser:

    @staticmethod
    def using(nodeToChildrenFunction: "Function"["T", Iterable["T"]]) -> "TreeTraverser"["T"]:
        """
        Returns a tree traverser that uses the given function to navigate from a node to its children.
        This is useful if the function instance already exists, or so that you can supply a lambda
        expressions. If those circumstances don't apply, you probably don't need to use this; subclass
        `TreeTraverser` and implement its .children method directly.

        Since
        - 20.0

        Deprecated
        - Use com.google.common.graph.Traverser.forTree instead. If you are using a
            lambda, these methods have exactly the same signature.
        """
        ...


    def children(self, root: "T") -> Iterable["T"]:
        """
        Returns the children of the specified node. Must not contain null.
        """
        ...


    def preOrderTraversal(self, root: "T") -> "FluentIterable"["T"]:
        """
        Returns an unmodifiable iterable over the nodes in a tree structure, using pre-order traversal.
        That is, each node's subtrees are traversed after the node itself is returned.
        
        No guarantees are made about the behavior of the traversal when nodes change while iteration
        is in progress or when the iterators generated by .children are advanced.

        Deprecated
        - Use com.google.common.graph.Traverser.depthFirstPreOrder instead, which has
            the same behavior.
        """
        ...


    def postOrderTraversal(self, root: "T") -> "FluentIterable"["T"]:
        """
        Returns an unmodifiable iterable over the nodes in a tree structure, using post-order
        traversal. That is, each node's subtrees are traversed before the node itself is returned.
        
        No guarantees are made about the behavior of the traversal when nodes change while iteration
        is in progress or when the iterators generated by .children are advanced.

        Deprecated
        - Use com.google.common.graph.Traverser.depthFirstPostOrder instead, which
            has the same behavior.
        """
        ...


    def breadthFirstTraversal(self, root: "T") -> "FluentIterable"["T"]:
        """
        Returns an unmodifiable iterable over the nodes in a tree structure, using breadth-first
        traversal. That is, all the nodes of depth 0 are returned, then depth 1, then 2, and so on.
        
        No guarantees are made about the behavior of the traversal when nodes change while iteration
        is in progress or when the iterators generated by .children are advanced.

        Deprecated
        - Use com.google.common.graph.Traverser.breadthFirst instead, which has the
            same behavior.
        """
        ...



"""
Python module generated from Java source file org.bukkit.advancement.AdvancementRequirements

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.advancement import *
from typing import Any, Callable, Iterable, Tuple


class AdvancementRequirements:
    """
    The list of requirements for the advancement.
    
    Requirements are complimentary to criteria. They are just lists that contain
    more lists, which in turn contains strings that equal the names of the
    criteria. Ultimately defining the logic around how criteria are completed in
    order to grant the advancement.

    See
    - <a href=https://www.minecraftforum.net/forums/minecraft-java-edition/redstone-discussion-and/commands-command-blocks-and/2809368-1-12-custom-advancements-aka-achievements.Requirements>Advancement Requirements</a>
    """

    def getRequirements(self) -> list["AdvancementRequirement"]:
        """
        Get all the requirements present in this advancement.

        Returns
        - an unmodifiable copy of all requirements.
        """
        ...



"""
Python module generated from Java source file java.util.NavigableSet

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util import *
from typing import Any, Callable, Iterable, Tuple


class NavigableSet(SortedSet):
    """
    A SortedSet extended with navigation methods reporting
    closest matches for given search targets. Methods .lower,
    .floor, .ceiling, and .higher return elements
    respectively less than, less than or equal, greater than or equal,
    and greater than a given element, returning `null` if there
    is no such element.
    
    A `NavigableSet` may be accessed and traversed in either
    ascending or descending order.  The .descendingSet method
    returns a view of the set with the senses of all relational and
    directional methods inverted. The performance of ascending
    operations and views is likely to be faster than that of descending
    ones.  This interface additionally defines methods .pollFirst and .pollLast that return and remove the lowest
    and highest element, if one exists, else returning `null`.
    Methods
    .subSet(Object, boolean, Object, boolean) subSet(E, boolean, E, boolean),
    .headSet(Object, boolean) headSet(E, boolean), and
    .tailSet(Object, boolean) tailSet(E, boolean)
    differ from the like-named `SortedSet` methods in accepting
    additional arguments describing whether lower and upper bounds are
    inclusive versus exclusive.  Subsets of any `NavigableSet`
    must implement the `NavigableSet` interface.
    
    The return values of navigation methods may be ambiguous in
    implementations that permit `null` elements. However, even
    in this case the result can be disambiguated by checking
    `contains(null)`. To avoid such issues, implementations of
    this interface are encouraged to *not* permit insertion of
    `null` elements. (Note that sorted sets of Comparable elements intrinsically do not permit `null`.)
    
    Methods
    .subSet(Object, Object) subSet(E, E),
    .headSet(Object) headSet(E), and
    .tailSet(Object) tailSet(E)
    are specified to return `SortedSet` to allow existing
    implementations of `SortedSet` to be compatibly retrofitted to
    implement `NavigableSet`, but extensions and implementations
    of this interface are encouraged to override these methods to return
    `NavigableSet`.
    
    This interface is a member of the
    <a href="/java.base/java/util/package-summary.html#CollectionsFramework">
    Java Collections Framework</a>.
    
    Type `<E>`: the type of elements maintained by this set

    Author(s)
    - Josh Bloch

    Since
    - 1.6
    """

    def lower(self, e: "E") -> "E":
        """
        Returns the greatest element in this set strictly less than the
        given element, or `null` if there is no such element.

        Arguments
        - e: the value to match

        Returns
        - the greatest element less than `e`,
                or `null` if there is no such element

        Raises
        - ClassCastException: if the specified element cannot be
                compared with the elements currently in the set
        - NullPointerException: if the specified element is null
                and this set does not permit null elements
        """
        ...


    def floor(self, e: "E") -> "E":
        """
        Returns the greatest element in this set less than or equal to
        the given element, or `null` if there is no such element.

        Arguments
        - e: the value to match

        Returns
        - the greatest element less than or equal to `e`,
                or `null` if there is no such element

        Raises
        - ClassCastException: if the specified element cannot be
                compared with the elements currently in the set
        - NullPointerException: if the specified element is null
                and this set does not permit null elements
        """
        ...


    def ceiling(self, e: "E") -> "E":
        """
        Returns the least element in this set greater than or equal to
        the given element, or `null` if there is no such element.

        Arguments
        - e: the value to match

        Returns
        - the least element greater than or equal to `e`,
                or `null` if there is no such element

        Raises
        - ClassCastException: if the specified element cannot be
                compared with the elements currently in the set
        - NullPointerException: if the specified element is null
                and this set does not permit null elements
        """
        ...


    def higher(self, e: "E") -> "E":
        """
        Returns the least element in this set strictly greater than the
        given element, or `null` if there is no such element.

        Arguments
        - e: the value to match

        Returns
        - the least element greater than `e`,
                or `null` if there is no such element

        Raises
        - ClassCastException: if the specified element cannot be
                compared with the elements currently in the set
        - NullPointerException: if the specified element is null
                and this set does not permit null elements
        """
        ...


    def pollFirst(self) -> "E":
        """
        Retrieves and removes the first (lowest) element,
        or returns `null` if this set is empty.

        Returns
        - the first element, or `null` if this set is empty
        """
        ...


    def pollLast(self) -> "E":
        """
        Retrieves and removes the last (highest) element,
        or returns `null` if this set is empty.

        Returns
        - the last element, or `null` if this set is empty
        """
        ...


    def iterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this set, in ascending order.

        Returns
        - an iterator over the elements in this set, in ascending order
        """
        ...


    def descendingSet(self) -> "NavigableSet"["E"]:
        """
        Returns a reverse order view of the elements contained in this set.
        The descending set is backed by this set, so changes to the set are
        reflected in the descending set, and vice-versa.  If either set is
        modified while an iteration over either set is in progress (except
        through the iterator's own `remove` operation), the results of
        the iteration are undefined.
        
        The returned set has an ordering equivalent to
        Collections.reverseOrder(Comparator) Collections.reverseOrder`(comparator())`.
        The expression `s.descendingSet().descendingSet()` returns a
        view of `s` essentially equivalent to `s`.

        Returns
        - a reverse order view of this set
        """
        ...


    def descendingIterator(self) -> Iterator["E"]:
        """
        Returns an iterator over the elements in this set, in descending order.
        Equivalent in effect to `descendingSet().iterator()`.

        Returns
        - an iterator over the elements in this set, in descending order
        """
        ...


    def subSet(self, fromElement: "E", fromInclusive: bool, toElement: "E", toInclusive: bool) -> "NavigableSet"["E"]:
        """
        Returns a view of the portion of this set whose elements range from
        `fromElement` to `toElement`.  If `fromElement` and
        `toElement` are equal, the returned set is empty unless `fromInclusive` and `toInclusive` are both True.  The returned set
        is backed by this set, so changes in the returned set are reflected in
        this set, and vice-versa.  The returned set supports all optional set
        operations that this set supports.
        
        The returned set will throw an `IllegalArgumentException`
        on an attempt to insert an element outside its range.

        Arguments
        - fromElement: low endpoint of the returned set
        - fromInclusive: `True` if the low endpoint
               is to be included in the returned view
        - toElement: high endpoint of the returned set
        - toInclusive: `True` if the high endpoint
               is to be included in the returned view

        Returns
        - a view of the portion of this set whose elements range from
                `fromElement`, inclusive, to `toElement`, exclusive

        Raises
        - ClassCastException: if `fromElement` and
                `toElement` cannot be compared to one another using this
                set's comparator (or, if the set has no comparator, using
                natural ordering).  Implementations may, but are not required
                to, throw this exception if `fromElement` or
                `toElement` cannot be compared to elements currently in
                the set.
        - NullPointerException: if `fromElement` or
                `toElement` is null and this set does
                not permit null elements
        - IllegalArgumentException: if `fromElement` is
                greater than `toElement`; or if this set itself
                has a restricted range, and `fromElement` or
                `toElement` lies outside the bounds of the range.
        """
        ...


    def headSet(self, toElement: "E", inclusive: bool) -> "NavigableSet"["E"]:
        """
        Returns a view of the portion of this set whose elements are less than
        (or equal to, if `inclusive` is True) `toElement`.  The
        returned set is backed by this set, so changes in the returned set are
        reflected in this set, and vice-versa.  The returned set supports all
        optional set operations that this set supports.
        
        The returned set will throw an `IllegalArgumentException`
        on an attempt to insert an element outside its range.

        Arguments
        - toElement: high endpoint of the returned set
        - inclusive: `True` if the high endpoint
               is to be included in the returned view

        Returns
        - a view of the portion of this set whose elements are less than
                (or equal to, if `inclusive` is True) `toElement`

        Raises
        - ClassCastException: if `toElement` is not compatible
                with this set's comparator (or, if the set has no comparator,
                if `toElement` does not implement Comparable).
                Implementations may, but are not required to, throw this
                exception if `toElement` cannot be compared to elements
                currently in the set.
        - NullPointerException: if `toElement` is null and
                this set does not permit null elements
        - IllegalArgumentException: if this set itself has a
                restricted range, and `toElement` lies outside the
                bounds of the range
        """
        ...


    def tailSet(self, fromElement: "E", inclusive: bool) -> "NavigableSet"["E"]:
        """
        Returns a view of the portion of this set whose elements are greater
        than (or equal to, if `inclusive` is True) `fromElement`.
        The returned set is backed by this set, so changes in the returned set
        are reflected in this set, and vice-versa.  The returned set supports
        all optional set operations that this set supports.
        
        The returned set will throw an `IllegalArgumentException`
        on an attempt to insert an element outside its range.

        Arguments
        - fromElement: low endpoint of the returned set
        - inclusive: `True` if the low endpoint
               is to be included in the returned view

        Returns
        - a view of the portion of this set whose elements are greater
                than or equal to `fromElement`

        Raises
        - ClassCastException: if `fromElement` is not compatible
                with this set's comparator (or, if the set has no comparator,
                if `fromElement` does not implement Comparable).
                Implementations may, but are not required to, throw this
                exception if `fromElement` cannot be compared to elements
                currently in the set.
        - NullPointerException: if `fromElement` is null
                and this set does not permit null elements
        - IllegalArgumentException: if this set itself has a
                restricted range, and `fromElement` lies outside the
                bounds of the range
        """
        ...


    def subSet(self, fromElement: "E", toElement: "E") -> "SortedSet"["E"]:
        """
        
        
        Equivalent to `subSet(fromElement, True, toElement, False)`.

        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def headSet(self, toElement: "E") -> "SortedSet"["E"]:
        """
        
        
        Equivalent to `headSet(toElement, False)`.

        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...


    def tailSet(self, fromElement: "E") -> "SortedSet"["E"]:
        """
        
        
        Equivalent to `tailSet(fromElement, True)`.

        Raises
        - ClassCastException: 
        - NullPointerException: 
        - IllegalArgumentException: 
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Steerable

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Steerable(Animals):
    """
    Represents an entity which may be saddled, ridden and steered using an item.
    """

    def hasSaddle(self) -> bool:
        """
        Check if the pig has a saddle.

        Returns
        - if the pig has been saddled.
        """
        ...


    def setSaddle(self, saddled: bool) -> None:
        """
        Sets if the pig has a saddle or not

        Arguments
        - saddled: set if the pig has a saddle or not.
        """
        ...


    def getBoostTicks(self) -> int:
        """
        Get the time in ticks this entity's movement is being increased.
        
        Movement speed is often increased as a result of using the
        .getSteerMaterial().

        Returns
        - the current boost ticks
        """
        ...


    def setBoostTicks(self, ticks: int) -> None:
        """
        Set the time in ticks this entity's movement will be increased.
        
        This will reset the current boost ticks to 0
        (.getCurrentBoostTicks()).

        Arguments
        - ticks: the boost time
        """
        ...


    def getCurrentBoostTicks(self) -> int:
        """
        Get the time in ticks this entity's movement has been increased as of the
        most recent boost.
        
        Current boost ticks will never be > .getBoostTicks().

        Returns
        - the current boost ticks
        """
        ...


    def setCurrentBoostTicks(self, ticks: int) -> None:
        """
        Set the time in ticks this entity's movement has been increased relative
        to the most recent boost.

        Arguments
        - ticks: the current boost ticks. Must be >= 0 and <=
        .getBoostTicks()
        """
        ...


    def getSteerMaterial(self) -> "Material":
        """
        Get the material used to steer this entity when ridden by a player.

        Returns
        - the lure material
        """
        ...



"""
Python module generated from Java source file org.bukkit.command.Command

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import ImmutableList
from java.util import Collections
from org.bukkit import Bukkit
from org.bukkit import ChatColor
from org.bukkit import GameRule
from org.bukkit import Location
from org.bukkit import Server
from org.bukkit.command import *
from org.bukkit.entity import Player
from org.bukkit.entity.minecart import CommandMinecart
from org.bukkit.permissions import Permissible
from org.bukkit.plugin import PluginDescriptionFile
from org.bukkit.util import StringUtil
from typing import Any, Callable, Iterable, Tuple


class Command:
    """
    Represents a Command, which executes various tasks upon user input
    """

    def execute(self, sender: "CommandSender", commandLabel: str, args: list[str]) -> bool:
        """
        Executes the command, returning its success

        Arguments
        - sender: Source object which is executing this command
        - commandLabel: The alias of the command used
        - args: All arguments passed to the command, split via ' '

        Returns
        - True if the command was successful, otherwise False
        """
        ...


    def tabComplete(self, sender: "CommandSender", alias: str, args: list[str]) -> list[str]:
        """
        Executed on tab completion for this command, returning a list of
        options the player can tab through.

        Arguments
        - sender: Source object which is executing this command
        - alias: the alias being used
        - args: All arguments passed to the command, split via ' '

        Returns
        - a list of tab-completions for the specified arguments. This
            will never be null. List may be immutable.

        Raises
        - IllegalArgumentException: if sender, alias, or args is null
        """
        ...


    def tabComplete(self, sender: "CommandSender", alias: str, args: list[str], location: "Location") -> list[str]:
        """
        Executed on tab completion for this command, returning a list of
        options the player can tab through.

        Arguments
        - sender: Source object which is executing this command
        - alias: the alias being used
        - args: All arguments passed to the command, split via ' '
        - location: The position looked at by the sender, or null if none

        Returns
        - a list of tab-completions for the specified arguments. This
            will never be null. List may be immutable.

        Raises
        - IllegalArgumentException: if sender, alias, or args is null
        """
        ...


    def getName(self) -> str:
        """
        Returns the name of this command

        Returns
        - Name of this command
        """
        ...


    def setName(self, name: str) -> bool:
        """
        Sets the name of this command.
        
        May only be used before registering the command.
        Will return True if the new name is set, and False
        if the command has already been registered.

        Arguments
        - name: New command name

        Returns
        - returns True if the name change happened instantly or False if
            the command was already registered
        """
        ...


    def getPermission(self) -> str:
        """
        Gets the permission required by users to be able to perform this
        command

        Returns
        - Permission name, or null if none
        """
        ...


    def setPermission(self, permission: str) -> None:
        """
        Sets the permission required by users to be able to perform this
        command

        Arguments
        - permission: Permission name or null
        """
        ...


    def testPermission(self, target: "CommandSender") -> bool:
        """
        Tests the given CommandSender to see if they can perform this
        command.
        
        If they do not have permission, they will be informed that they cannot
        do this.

        Arguments
        - target: User to test

        Returns
        - True if they can use it, otherwise False
        """
        ...


    def testPermissionSilent(self, target: "CommandSender") -> bool:
        """
        Tests the given CommandSender to see if they can perform this
        command.
        
        No error is sent to the sender.

        Arguments
        - target: User to test

        Returns
        - True if they can use it, otherwise False
        """
        ...


    def getLabel(self) -> str:
        """
        Returns the label for this command

        Returns
        - Label of this command
        """
        ...


    def setLabel(self, name: str) -> bool:
        """
        Sets the label of this command.
        
        May only be used before registering the command.
        Will return True if the new name is set, and False
        if the command has already been registered.

        Arguments
        - name: The command's name

        Returns
        - returns True if the name change happened instantly or False if
            the command was already registered
        """
        ...


    def register(self, commandMap: "CommandMap") -> bool:
        """
        Registers this command to a CommandMap.
        Once called it only allows changes the registered CommandMap

        Arguments
        - commandMap: the CommandMap to register this command to

        Returns
        - True if the registration was successful (the current registered
            CommandMap was the passed CommandMap or null) False otherwise
        """
        ...


    def unregister(self, commandMap: "CommandMap") -> bool:
        """
        Unregisters this command from the passed CommandMap applying any
        outstanding changes

        Arguments
        - commandMap: the CommandMap to unregister

        Returns
        - True if the unregistration was successful (the current
            registered CommandMap was the passed CommandMap or null) False
            otherwise
        """
        ...


    def isRegistered(self) -> bool:
        """
        Returns the current registered state of this command

        Returns
        - True if this command is currently registered False otherwise
        """
        ...


    def getAliases(self) -> list[str]:
        """
        Returns a list of active aliases of this command

        Returns
        - List of aliases
        """
        ...


    def getPermissionMessage(self) -> str:
        """
        Returns a message to be displayed on a failed permission check for this
        command

        Returns
        - Permission check failed message

        Deprecated
        - permission messages have not worked for player-executed
        commands since 1.13 as clients without permission to execute a command
        are unaware of its existence and therefore will not send an unknown
        command execution to the server. This message will only ever be shown to
        consoles or when this command is executed with
        Bukkit.dispatchCommand(CommandSender, String).
        """
        ...


    def getDescription(self) -> str:
        """
        Gets a brief description of this command

        Returns
        - Description of this command
        """
        ...


    def getUsage(self) -> str:
        """
        Gets an example usage of this command

        Returns
        - One or more example usages
        """
        ...


    def setAliases(self, aliases: list[str]) -> "Command":
        """
        Sets the list of aliases to request on registration for this command.
        This is not effective outside of defining aliases in the PluginDescriptionFile.getCommands() (under the
        ``aliases`' node) is equivalent to this method.

        Arguments
        - aliases: aliases to register to this command

        Returns
        - this command object, for chaining
        """
        ...


    def setDescription(self, description: str) -> "Command":
        """
        Sets a brief description of this command. Defining a description in the
        PluginDescriptionFile.getCommands() (under the
        ``description`' node) is equivalent to this method.

        Arguments
        - description: new command description

        Returns
        - this command object, for chaining
        """
        ...


    def setPermissionMessage(self, permissionMessage: str) -> "Command":
        """
        Sets the message sent when a permission check fails

        Arguments
        - permissionMessage: new permission message, null to indicate
            default message, or an empty string to indicate no message

        Returns
        - this command object, for chaining

        Deprecated
        - permission messages have not worked for player-executed
        commands since 1.13 as clients without permission to execute a command
        are unaware of its existence and therefore will not send an unknown
        command execution to the server. This message will only ever be shown to
        consoles or when this command is executed with
        Bukkit.dispatchCommand(CommandSender, String).
        """
        ...


    def setUsage(self, usage: str) -> "Command":
        """
        Sets the example usage of this command

        Arguments
        - usage: new example usage

        Returns
        - this command object, for chaining
        """
        ...


    @staticmethod
    def broadcastCommandMessage(source: "CommandSender", message: str) -> None:
        ...


    @staticmethod
    def broadcastCommandMessage(source: "CommandSender", message: str, sendToSource: bool) -> None:
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file java.nio.file.attribute.BasicFileAttributeView

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.nio.file.attribute import *
from typing import Any, Callable, Iterable, Tuple


class BasicFileAttributeView(FileAttributeView):

    def name(self) -> str:
        """
        Returns the name of the attribute view. Attribute views of this type
        have the name `"basic"`.
        """
        ...


    def readAttributes(self) -> "BasicFileAttributes":
        """
        Reads the basic file attributes as a bulk operation.
        
         It is implementation specific if all file attributes are read as an
        atomic operation with respect to other file system operations.

        Returns
        - the file attributes

        Raises
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, a security manager is
                 installed, its SecurityManager.checkRead(String) checkRead
                 method is invoked to check read access to the file
        """
        ...


    def setTimes(self, lastModifiedTime: "FileTime", lastAccessTime: "FileTime", createTime: "FileTime") -> None:
        """
        Updates any or all of the file's last modified time, last access time,
        and create time attributes.
        
         This method updates the file's timestamp attributes. The values are
        converted to the epoch and precision supported by the file system.
        Converting from finer to coarser granularities result in precision loss.
        The behavior of this method when attempting to set a timestamp that is
        not supported or to a value that is outside the range supported by the
        underlying file store is not defined. It may or not fail by throwing an
        `IOException`.
        
         If any of the `lastModifiedTime`, `lastAccessTime`,
        or `createTime` parameters has the value `null` then the
        corresponding timestamp is not changed. An implementation may require to
        read the existing values of the file attributes when only some, but not
        all, of the timestamp attributes are updated. Consequently, this method
        may not be an atomic operation with respect to other file system
        operations. Reading and re-writing existing values may also result in
        precision loss. If all of the `lastModifiedTime`, `lastAccessTime` and `createTime` parameters are `null` then
        this method has no effect.
        
         **Usage Example:**
        Suppose we want to change a file's last access time.
        ```
           Path path = ...
           FileTime time = ...
           Files.getFileAttributeView(path, BasicFileAttributeView.class).setTimes(null, time, null);
        ```

        Arguments
        - lastModifiedTime: the new last modified time, or `null` to not change the
                 value
        - lastAccessTime: the last access time, or `null` to not change the value
        - createTime: the file's create time, or `null` to not change the value

        Raises
        - IOException: if an I/O error occurs
        - SecurityException: In the case of the default provider, a security manager is
                 installed, its  SecurityManager.checkWrite(String) checkWrite
                 method is invoked to check write access to the file

        See
        - java.nio.file.Files.setLastModifiedTime
        """
        ...



"""
Python module generated from Java source file com.google.common.io.ReaderInputStream

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.io import *
from com.google.common.primitives import UnsignedBytes
from java.io import IOException
from java.io import InputStream
from java.io import Reader
from java.nio.charset import Charset
from java.nio.charset import CharsetEncoder
from java.nio.charset import CoderResult
from java.nio.charset import CodingErrorAction
from java.util import Arrays
from typing import Any, Callable, Iterable, Tuple


class ReaderInputStream(InputStream):
    """
    An InputStream that converts characters from a Reader into bytes using an
    arbitrary Charset.
    
    This is an alternative to copying the data to an `OutputStream` via a `Writer`,
    which is necessarily blocking. By implementing an `InputStream` it allows consumers to
    "pull" as much data as they can handle, which is more convenient when dealing with flow
    controlled, async APIs.

    Author(s)
    - Chris Nokleberg
    """

    def close(self) -> None:
        ...


    def read(self) -> int:
        ...


    def read(self, b: list[int], off: int, len: int) -> int:
        ...



"""
Python module generated from Java source file org.bukkit.entity.Creaking

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Creaking(Monster):
    """
    Represents a Creaking.
    """

    def getHome(self) -> "Location":
        """
        Gets the home location for this Creaking (ie where its corresponding
        org.bukkit.block.CreakingHeart can be).

        Returns
        - the location of the home.
        """
        ...


    def setHome(self, location: "Location") -> None:
        """
        Sets the home location for this Creaking.

        Arguments
        - location: the location of the home.
        """
        ...


    def activate(self, player: "Player") -> None:
        """
        Activate this Creaking to target and follow a player.

        Arguments
        - player: the target.
        """
        ...


    def deactivate(self) -> None:
        """
        Deactivate this Creaking from the current target player.
        """
        ...


    def isActive(self) -> bool:
        """
        Gets if this Creaking is active.

        Returns
        - True if is active.
        """
        ...



"""
Python module generated from Java source file java.util.concurrent.atomic.AtomicLong

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.lang.invoke import VarHandle
from java.util.concurrent.atomic import *
from java.util.function import LongBinaryOperator
from java.util.function import LongUnaryOperator
from jdk.internal.misc import Unsafe
from typing import Any, Callable, Iterable, Tuple


class AtomicLong(Number, Serializable):
    """
    A `long` value that may be updated atomically.  See the
    VarHandle specification for descriptions of the properties
    of atomic accesses. An `AtomicLong` is used in applications
    such as atomically incremented sequence numbers, and cannot be used
    as a replacement for a java.lang.Long. However, this class
    does extend `Number` to allow uniform access by tools and
    utilities that deal with numerically-based classes.

    Author(s)
    - Doug Lea

    Since
    - 1.5
    """

    def __init__(self, initialValue: int):
        """
        Creates a new AtomicLong with the given initial value.

        Arguments
        - initialValue: the initial value
        """
        ...


    def __init__(self):
        """
        Creates a new AtomicLong with initial value `0`.
        """
        ...


    def get(self) -> int:
        """
        Returns the current value,
        with memory effects as specified by VarHandle.getVolatile.

        Returns
        - the current value
        """
        ...


    def set(self, newValue: int) -> None:
        """
        Sets the value to `newValue`,
        with memory effects as specified by VarHandle.setVolatile.

        Arguments
        - newValue: the new value
        """
        ...


    def lazySet(self, newValue: int) -> None:
        """
        Sets the value to `newValue`,
        with memory effects as specified by VarHandle.setRelease.

        Arguments
        - newValue: the new value

        Since
        - 1.6
        """
        ...


    def getAndSet(self, newValue: int) -> int:
        """
        Atomically sets the value to `newValue` and returns the old value,
        with memory effects as specified by VarHandle.getAndSet.

        Arguments
        - newValue: the new value

        Returns
        - the previous value
        """
        ...


    def compareAndSet(self, expectedValue: int, newValue: int) -> bool:
        """
        Atomically sets the value to `newValue`
        if the current value `== expectedValue`,
        with memory effects as specified by VarHandle.compareAndSet.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - `True` if successful. False return indicates that
        the actual value was not equal to the expected value.
        """
        ...


    def weakCompareAndSet(self, expectedValue: int, newValue: int) -> bool:
        """
        Possibly atomically sets the value to `newValue`
        if the current value `== expectedValue`,
        with memory effects as specified by VarHandle.weakCompareAndSetPlain.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - `True` if successful

        See
        - .weakCompareAndSetPlain

        Deprecated
        - This method has plain memory effects but the method
        name implies volatile memory effects (see methods such as
        .compareAndExchange and .compareAndSet).  To avoid
        confusion over plain or volatile memory effects it is recommended that
        the method .weakCompareAndSetPlain be used instead.
        """
        ...


    def weakCompareAndSetPlain(self, expectedValue: int, newValue: int) -> bool:
        """
        Possibly atomically sets the value to `newValue`
        if the current value `== expectedValue`,
        with memory effects as specified by VarHandle.weakCompareAndSetPlain.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - `True` if successful

        Since
        - 9
        """
        ...


    def getAndIncrement(self) -> int:
        """
        Atomically increments the current value,
        with memory effects as specified by VarHandle.getAndAdd.
        
        Equivalent to `getAndAdd(1)`.

        Returns
        - the previous value
        """
        ...


    def getAndDecrement(self) -> int:
        """
        Atomically decrements the current value,
        with memory effects as specified by VarHandle.getAndAdd.
        
        Equivalent to `getAndAdd(-1)`.

        Returns
        - the previous value
        """
        ...


    def getAndAdd(self, delta: int) -> int:
        """
        Atomically adds the given value to the current value,
        with memory effects as specified by VarHandle.getAndAdd.

        Arguments
        - delta: the value to add

        Returns
        - the previous value
        """
        ...


    def incrementAndGet(self) -> int:
        """
        Atomically increments the current value,
        with memory effects as specified by VarHandle.getAndAdd.
        
        Equivalent to `addAndGet(1)`.

        Returns
        - the updated value
        """
        ...


    def decrementAndGet(self) -> int:
        """
        Atomically decrements the current value,
        with memory effects as specified by VarHandle.getAndAdd.
        
        Equivalent to `addAndGet(-1)`.

        Returns
        - the updated value
        """
        ...


    def addAndGet(self, delta: int) -> int:
        """
        Atomically adds the given value to the current value,
        with memory effects as specified by VarHandle.getAndAdd.

        Arguments
        - delta: the value to add

        Returns
        - the updated value
        """
        ...


    def getAndUpdate(self, updateFunction: "LongUnaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the current value with the results of
        applying the given function, returning the previous value. The
        function should be side-effect-free, since it may be re-applied
        when attempted updates fail due to contention among threads.

        Arguments
        - updateFunction: a side-effect-free function

        Returns
        - the previous value

        Since
        - 1.8
        """
        ...


    def updateAndGet(self, updateFunction: "LongUnaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the current value with the results of
        applying the given function, returning the updated value. The
        function should be side-effect-free, since it may be re-applied
        when attempted updates fail due to contention among threads.

        Arguments
        - updateFunction: a side-effect-free function

        Returns
        - the updated value

        Since
        - 1.8
        """
        ...


    def getAndAccumulate(self, x: int, accumulatorFunction: "LongBinaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the current value with the results of
        applying the given function to the current and given values,
        returning the previous value. The function should be
        side-effect-free, since it may be re-applied when attempted
        updates fail due to contention among threads.  The function is
        applied with the current value as its first argument, and the
        given update as the second argument.

        Arguments
        - x: the update value
        - accumulatorFunction: a side-effect-free function of two arguments

        Returns
        - the previous value

        Since
        - 1.8
        """
        ...


    def accumulateAndGet(self, x: int, accumulatorFunction: "LongBinaryOperator") -> int:
        """
        Atomically updates (with memory effects as specified by VarHandle.compareAndSet) the current value with the results of
        applying the given function to the current and given values,
        returning the updated value. The function should be
        side-effect-free, since it may be re-applied when attempted
        updates fail due to contention among threads.  The function is
        applied with the current value as its first argument, and the
        given update as the second argument.

        Arguments
        - x: the update value
        - accumulatorFunction: a side-effect-free function of two arguments

        Returns
        - the updated value

        Since
        - 1.8
        """
        ...


    def toString(self) -> str:
        """
        Returns the String representation of the current value.

        Returns
        - the String representation of the current value
        """
        ...


    def intValue(self) -> int:
        """
        Returns the current value of this `AtomicLong` as an `int`
        after a narrowing primitive conversion,
        with memory effects as specified by VarHandle.getVolatile.

        Unknown Tags
        - 5.1.3 Narrowing Primitive Conversion
        """
        ...


    def longValue(self) -> int:
        """
        Returns the current value of this `AtomicLong` as a `long`,
        with memory effects as specified by VarHandle.getVolatile.
        Equivalent to .get().
        """
        ...


    def floatValue(self) -> float:
        """
        Returns the current value of this `AtomicLong` as a `float`
        after a widening primitive conversion,
        with memory effects as specified by VarHandle.getVolatile.

        Unknown Tags
        - 5.1.2 Widening Primitive Conversion
        """
        ...


    def doubleValue(self) -> float:
        """
        Returns the current value of this `AtomicLong` as a `double`
        after a widening primitive conversion,
        with memory effects as specified by VarHandle.getVolatile.

        Unknown Tags
        - 5.1.2 Widening Primitive Conversion
        """
        ...


    def getPlain(self) -> int:
        """
        Returns the current value, with memory semantics of reading as if the
        variable was declared non-`volatile`.

        Returns
        - the value

        Since
        - 9
        """
        ...


    def setPlain(self, newValue: int) -> None:
        """
        Sets the value to `newValue`, with memory semantics
        of setting as if the variable was declared non-`volatile`
        and non-`final`.

        Arguments
        - newValue: the new value

        Since
        - 9
        """
        ...


    def getOpaque(self) -> int:
        """
        Returns the current value,
        with memory effects as specified by VarHandle.getOpaque.

        Returns
        - the value

        Since
        - 9
        """
        ...


    def setOpaque(self, newValue: int) -> None:
        """
        Sets the value to `newValue`,
        with memory effects as specified by VarHandle.setOpaque.

        Arguments
        - newValue: the new value

        Since
        - 9
        """
        ...


    def getAcquire(self) -> int:
        """
        Returns the current value,
        with memory effects as specified by VarHandle.getAcquire.

        Returns
        - the value

        Since
        - 9
        """
        ...


    def setRelease(self, newValue: int) -> None:
        """
        Sets the value to `newValue`,
        with memory effects as specified by VarHandle.setRelease.

        Arguments
        - newValue: the new value

        Since
        - 9
        """
        ...


    def compareAndExchange(self, expectedValue: int, newValue: int) -> int:
        """
        Atomically sets the value to `newValue` if the current value,
        referred to as the *witness value*, `== expectedValue`,
        with memory effects as specified by
        VarHandle.compareAndExchange.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - the witness value, which will be the same as the
        expected value if successful

        Since
        - 9
        """
        ...


    def compareAndExchangeAcquire(self, expectedValue: int, newValue: int) -> int:
        """
        Atomically sets the value to `newValue` if the current value,
        referred to as the *witness value*, `== expectedValue`,
        with memory effects as specified by
        VarHandle.compareAndExchangeAcquire.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - the witness value, which will be the same as the
        expected value if successful

        Since
        - 9
        """
        ...


    def compareAndExchangeRelease(self, expectedValue: int, newValue: int) -> int:
        """
        Atomically sets the value to `newValue` if the current value,
        referred to as the *witness value*, `== expectedValue`,
        with memory effects as specified by
        VarHandle.compareAndExchangeRelease.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - the witness value, which will be the same as the
        expected value if successful

        Since
        - 9
        """
        ...


    def weakCompareAndSetVolatile(self, expectedValue: int, newValue: int) -> bool:
        """
        Possibly atomically sets the value to `newValue`
        if the current value `== expectedValue`,
        with memory effects as specified by
        VarHandle.weakCompareAndSet.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - `True` if successful

        Since
        - 9
        """
        ...


    def weakCompareAndSetAcquire(self, expectedValue: int, newValue: int) -> bool:
        """
        Possibly atomically sets the value to `newValue`
        if the current value `== expectedValue`,
        with memory effects as specified by
        VarHandle.weakCompareAndSetAcquire.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - `True` if successful

        Since
        - 9
        """
        ...


    def weakCompareAndSetRelease(self, expectedValue: int, newValue: int) -> bool:
        """
        Possibly atomically sets the value to `newValue`
        if the current value `== expectedValue`,
        with memory effects as specified by
        VarHandle.weakCompareAndSetRelease.

        Arguments
        - expectedValue: the expected value
        - newValue: the new value

        Returns
        - `True` if successful

        Since
        - 9
        """
        ...



"""
Python module generated from Java source file com.google.gson.internal.JavaVersion

Java source file obtained from artifact gson version 2.11.0

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.gson.internal import *
from typing import Any, Callable, Iterable, Tuple


class JavaVersion:
    """
    Utility to check the major Java version of the current JVM.
    """

    @staticmethod
    def getMajorJavaVersion() -> int:
        """
        Gets the major Java version

        Returns
        - the major Java version, i.e. '8' for Java 1.8, '9' for Java 9 etc.
        """
        ...


    @staticmethod
    def isJava9OrLater() -> bool:
        """
        Gets a boolean value depending if the application is running on Java 9 or later

        Returns
        - `True` if the application is running on Java 9 or later; and `False`
            otherwise.
        """
        ...



"""
Python module generated from Java source file org.bukkit.conversations.NumericPrompt

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.apache.commons.lang3.math import NumberUtils
from org.bukkit.conversations import *
from typing import Any, Callable, Iterable, Tuple


class NumericPrompt(ValidatingPrompt):
    """
    NumericPrompt is the base class for any prompt that requires a Number response from the user.
    """

    def __init__(self):
        ...



"""
Python module generated from Java source file com.google.common.hash.Murmur3_32HashFunction

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.hash import *
from com.google.common.primitives import Chars
from com.google.common.primitives import Ints
from com.google.common.primitives import Longs
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.errorprone.annotations import Immutable
from java.io import Serializable
from java.nio.charset import Charset
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Murmur3_32HashFunction(AbstractHashFunction, Serializable):
    """
    See MurmurHash3_x86_32 in <a
    href="https://github.com/aappleby/smhasher/blob/master/src/MurmurHash3.cpp">the C++
    implementation</a>.

    Author(s)
    - Kurt Alfred Kluever
    """

    def bits(self) -> int:
        ...


    def newHasher(self) -> "Hasher":
        ...


    def toString(self) -> str:
        ...


    def equals(self, object: "Object") -> bool:
        ...


    def hashCode(self) -> int:
        ...


    def hashInt(self, input: int) -> "HashCode":
        ...


    def hashLong(self, input: int) -> "HashCode":
        ...


    def hashUnencodedChars(self, input: "CharSequence") -> "HashCode":
        ...


    def hashString(self, input: "CharSequence", charset: "Charset") -> "HashCode":
        ...


    def hashBytes(self, input: list[int], off: int, len: int) -> "HashCode":
        ...
